# -*- coding: utf-8 -*-

"""Script for extracting patent literature from SureChEMBL."""

import logging
import os
import time
from typing import Tuple

import pandas as pd
from tqdm import tqdm

from pemt.constants import DATA_DIR, PATENT_DIR, VALID_CODES

# Selenium specific settings
try:
    from selenium import webdriver
except ImportError:
    raise ValueError("please install selenium before running this script")

from selenium.common.exceptions import NoSuchElementException, TimeoutException
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait

chrome_options = Options()
chrome_options.add_argument("--headless")
chrome_options.add_argument("--window-size=1920x1080")
chrome_options.add_argument("--disable-notifications")
chrome_options.add_argument("--no-sandbox")
chrome_options.add_argument("--verbose")
chrome_options.add_experimental_option(
    "prefs",
    {
        "download.prompt_for_download": False,
        "download.directory_upgrade": True,
        "safebrowsing_for_trusted_sources_enabled": False,
        "safebrowsing.enabled": False,
    },
)
chrome_options.add_argument("--disable-gpu")
chrome_options.add_argument("--disable-software-rasterizer")

logger = logging.getLogger("__name__")
logger.setLevel(logging.INFO)

"""Constant factors related to scraping"""

os.makedirs(f"{PATENT_DIR}", exist_ok=True)


def get_valid_patent_list(
    schembl_id: str, system: str, chrome_driver_path: str, year: int
) -> Tuple[set, int]:
    """Get valid patents from SureChEMBL based on their IPC criteria and time period.

    :param schembl_id: The SureChEMBL id of the compound.
    :param system: The OS on which the code is running. It can be either of these: linux, mac, window.
    :param chrome_driver_path: The path of the chrome driver is located.
    :param year: The cutt-off year for searching the patent documents
    """

    # Replace path to chrome driver (https://sites.google.com/a/chromium.org/chromedriver/home)
    driver = webdriver.Chrome(
        options=chrome_options,
        executable_path=chrome_driver_path,
    )

    # function to take care of downloading file
    driver.command_executor._commands["send_command"] = (
        "POST",
        "/session/$sessionId/chromium/send_command",
    )
    params = {
        "cmd": "Page.setDownloadBehavior",
        "params": {"behavior": "allow", "downloadPath": DATA_DIR},
    }
    driver.execute("send_command", params)

    system = system.lower()

    logger.debug("Getting page")
    driver.get(f"https://www.surechembl.org/chemical/{schembl_id}")
    logger.debug("Page done")

    time.sleep(8)

    # Go to patent tab
    try:
        patent_button = driver.find_element_by_xpath(
            "/html/body/div/div/div[2]/div/div/div[3]/div[2]/ul/li[3]"
        )
        patent_button.click()
    except NoSuchElementException:  # no patents found
        return set(), 0

    time.sleep(15)

    try:
        element_present = EC.presence_of_element_located(
            (By.ID, "patent-hits-container")
        )
        WebDriverWait(driver, 30).until(element_present)
    except TimeoutException:
        logger.info("Timed out waiting for page to load")

    # Get the link for opening patent table
    try:
        new_link = driver.find_element_by_xpath(
            "/html/body/div/div/div[2]/div/div/div[3]/div[2]/div[3]/div[3]/a"
        ).get_attribute("href")
    except NoSuchElementException:  # no patents found
        return set(), 0

    # Get total number of patents
    range_val = int(
        driver.find_element_by_xpath("//span[@class='total_hits_data']").text.replace(
            ",", ""
        )
    )

    driver.get(new_link)
    time.sleep(2)

    patent_info = set()

    logger.info(f"Looking into {range_val} patents for {schembl_id}")

    for patent_count in range(1, range_val + 1):
        for i in range(2, 52):  # max number of elements in each page
            try:
                if system not in ["linux", "mac"]:
                    ipc_num = driver.find_element_by_xpath(
                        f"/html/body/div[1]/div/div[2]/div[1]/div[2]/div/div[2]/table/tbody/tr[{i}]/td[4]/div[1]/table/tbody/tr/td[1]"
                    ).text
                else:
                    ipc_num = driver.find_element_by_xpath(
                        f"/html/body/div/div/div[2]/div[1]/div[2]/div/div[2]/table/tbody/tr[{i}]/td[4]/div[1]/table/tbody/tr/td[1]"
                    ).text
            except NoSuchElementException:  # No IPC code found
                continue

            if not ipc_num:  # cases where IPC number is missing are skipped
                continue

            # Filter based on codes
            code = ipc_num.split()[0]
            if code not in VALID_CODES:
                continue

            if system not in ["linux", "mac"]:
                patent_date = driver.find_element_by_xpath(
                    f"/html/body/div[1]/div/div[2]/div[1]/div[2]/div/div[2]/table/tbody/tr[{i}]/td[3]"
                ).text
            else:
                patent_date = driver.find_element_by_xpath(
                    f"/html/body/div/div/div[2]/div[1]/div[2]/div/div[2]/table/tbody/tr[{i}]/td[3]"
                ).text
            patent_year = int(patent_date.split("-")[0])

            # Filter based on patent year
            if patent_year < year:
                continue

            if system not in ["linux", "mac"]:
                patent_number = driver.find_element_by_xpath(
                    f"/html/body/div[1]/div/div[2]/div[1]/div[2]/div/div[2]/table/tbody/tr[{i}]/td[2]"
                ).text.split("\n")[1]
            else:
                patent_number = driver.find_element_by_xpath(
                    f"/html/body/div/div/div[2]/div[1]/div[2]/div/div[2]/table/tbody/tr[{i}]/td[2]"
                ).text.split("\n")[1]

            # Get assignee information
            if system not in ["linux", "mac"]:
                assignee = driver.find_element_by_xpath(
                    f"/html/body/div[1]/div/div[2]/div[1]/div[2]/div/div[2]/table/tbody/tr[{i}]/td[4]/div[1]/table/tbody/tr/td[2]/a"
                ).text
            else:
                assignee = driver.find_element_by_xpath(
                    f"/html/body/div/div/div[2]/div[1]/div[2]/div/div[2]/table/tbody/tr[{i}]/td[4]/div[1]/table/tbody/tr/td[2]/a"
                ).text
            patent_info.add((patent_number, patent_date, ipc_num, assignee))

        if range_val < 50:  # entry fits in 1 page
            break

        # Go to next page
        try:
            if patent_count == 1:
                nx_button_num = 2
            else:
                nx_button_num = 4
            if system not in ["linux", "mac"]:
                next_page = driver.find_element_by_xpath(
                    f"/html/body/div[1]/div/div[2]/div[1]/div[2]/div[1]/div[3]/div[2]/ul/li[{nx_button_num}]/a"
                ).get_attribute("href")
            else:
                next_page = driver.find_element_by_xpath(
                    f"/html/body/div/div/div[2]/div[1]/div[2]/div[1]/div[3]/div[2]/ul/li[{nx_button_num}]/a"
                ).get_attribute("href")
            driver.get(next_page)
            time.sleep(8)
        except NoSuchElementException:
            continue

    return patent_info, range_val


def extract_patent(
    analysis_name: str,
    chrome_driver_path: str,
    os_system: str = "linux",
    patent_year: int = 2000,
) -> pd.DataFrame:
    """Extract and store all valid patent document metadata.

    :param analysis_name: Name of the analysis.
    :param os_system: The OS on which the code is running. It can be either of these: linux, mac, window.
    :param chrome_driver_path: The path of the chrome driver is located.
    :param patent_year: The cutt-off year for searching the patent documents
    """
    df = pd.read_csv(
        f"{PATENT_DIR}/{analysis_name}_chemicals.tsv",
        sep="\t",
        dtype=str,
        usecols=["chembl", "schembl_id"],
    )

    if df.empty:
        return pd.DataFrame()

    os_system = os_system.lower()
    assert os_system in ["linux", "mac", "windows"]
    logger.warning(
        f"Currently running on {os_system} OS. Please change if this is not the case."
    )

    # Check for existing cache file
    if os.path.exists(f"{PATENT_DIR}/{analysis_name}_patent_data.tsv"):
        patent_df = pd.read_csv(
            f"{PATENT_DIR}/{analysis_name}_patent_data.tsv", sep="\t"
        )
    else:
        patent_df = pd.DataFrame(columns=["chembl", "surechembl"])

    cache_count = 0

    for chembl_id, surechembl_idx in tqdm(df.values, total=df.shape[0]):

        if pd.isna(surechembl_idx):
            continue

        _info_df = patent_df[
            (patent_df["chembl"] == chembl_id)
            & (patent_df["surechembl"] == surechembl_idx)
        ]

        if not _info_df.empty:
            continue

        cache_count += 1

        patent_info, total = get_valid_patent_list(
            schembl_id=surechembl_idx,
            system=os_system,
            chrome_driver_path=chrome_driver_path,
            year=patent_year,
        )

        if len(patent_info) == 0:
            patent_df = pd.concat(
                [
                    patent_df,
                    pd.DataFrame(
                        {
                            "chembl": chembl_id,
                            "surechembl": surechembl_idx,
                            "patent_id": "",
                            "date": "",
                            "ipc": "",
                            "assignee": "",
                        },
                        index=[0],
                    ),
                ],
                ignore_index=True,
            )
        else:
            for patent in patent_info:
                (pid, date, ipc_code, assignee) = patent
                patent_df = pd.concat(
                    [
                        patent_df,
                        pd.DataFrame(
                            {
                                "chembl": chembl_id,
                                "surechembl": surechembl_idx,
                                "patent_id": pid,
                                "date": date,
                                "ipc": ipc_code,
                                "assignee": assignee,
                            },
                            index=[0],
                        ),
                    ],
                    ignore_index=True,
                )

        if cache_count == 5:  # in case of internet issues
            patent_df.drop_duplicates(inplace=True)
            patent_df.to_csv(
                f"{PATENT_DIR}/{analysis_name}_patent_data.tsv", sep="\t", index=False
            )
            cache_count = 0

    patent_df.drop_duplicates(inplace=True)
    patent_df.dropna(subset=["patent_id", "date", "ipc"], inplace=True)
    patent_df.to_csv(
        f"{PATENT_DIR}/{analysis_name}_patent_data.tsv", sep="\t", index=False
    )
    return patent_df



# -*- coding: utf-8 -*-

"""Script for extracting experimental bioassay information from ChEMBL."""

import json
import logging
import os
from collections import defaultdict
from typing import List

import pandas as pd
from chembl_webresource_client.new_client import new_client
from tqdm import tqdm

from pemt.constants import MAPPER_DIR
from pemt.utils import hgnc_to_chembl, uniprot_to_chembl

logger = logging.getLogger(__name__)

# Change logging level for packages
chembl_logger = logging.getLogger("chembl_webresource_client")
chembl_logger.setLevel(logging.WARNING)

activity = new_client.activity

tqdm.pandas()


def get_chemical_overview(file_path: str) -> None:
    """Method to report incomplete information in the chemical enrichment.

    :param file_path: Path of the JSON file storing the gene and chemical information.
    """
    gene_chemical_dict = json.load(open(file_path))

    counter_dict = defaultdict(int)

    for gene, counter in tqdm(gene_chemical_dict.items()):
        counter_dict[gene] += len(counter)

    counter_dict = dict(
        sorted(counter_dict.items(), key=lambda item: item[1], reverse=True)
    )

    df = pd.DataFrame(counter_dict, index=[0]).transpose()
    value_count_dict = df[0].value_counts().to_dict()
    logger.warning(
        f"{value_count_dict[0]} genes found with no relevant chemical bioassay information."
    )


def target_to_chemical(
    chemical_mapping: dict,
    protein: str,
    protein_mapping: dict = None,
    is_uniprot: bool = False,
) -> List[dict]:
    """Method to retrieve bioactive chemicals, from proteins, based on biochemical/ functional bioassays.
    A chemical is considered active if it has a pChEMBL > 6.

    :param chemical_mapping: A dictionary mapping the UNIPROT identifiers to ChEMBL identifiers
    :param protein: The protein name or identifier
    :param protein_mapping: A dictionary mapping the HGNC symbols to UNIPROT identifiers.
    By default, the value is set to None.
    :param is_uniprot: Boolean indicating whether the protein is an HGNC symbol or UNIPROT identifier.
    If using UniProt ids for protein, set the value to "True" and the protein_mapping parameter can be omitted.
    If using HGNC symbols, then the protein mapping dictionary needs to be provided.
    """
    chemicals = []

    if not is_uniprot:
        try:
            assert protein_mapping is not None
        except AssertionError:
            raise ValueError(
                f"HGNC symbol given without passing the HGNC to UNIPROT mapping file. \
            Either pass the mapping file to hgnc_mapping variable or set the parameter is_uniprot=True"
            )
        target_chembl = hgnc_to_chembl(
            uniprot_mapper=protein_mapping,
            chemical_mapper=chemical_mapping,
            hgnc_symbol=protein,
        )
    else:
        target_chembl = uniprot_to_chembl(
            chemical_mapper=chemical_mapping, uniprot_id=protein
        )

    if not target_chembl:
        return chemicals

    prot_activity_data = activity.filter(
        target_chembl_id=target_chembl,
        assay_type_iregex="(B|F)",
    ).only(["pchembl_value", "molecule_chembl_id"])

    if len(prot_activity_data) < 1:
        return chemicals

    logger.debug(f"Analysing {len(prot_activity_data)} chemicals")
    for i in prot_activity_data:
        pchembl_val = i["pchembl_value"]

        if pd.isna(pchembl_val):
            continue

        if float(pchembl_val) < 6:
            continue

        chemicals.append(
            i["molecule_chembl_id"],
        )

    return chemicals


def extract_chemicals(
    analysis_name: str,
    gene_list: list = None,
    gene_file_path: str = None,
    file_separator: str = "comma",
    is_uniprot: bool = False,
    chembl_version: str = "30",
):
    """Enrich genes with chemical data from CheMBL bioassays.

    :param analysis_name: The name of the analysis you want to run. This name would be used to save the resultant file
    :param gene_list: The list of gene you want to extract chemicals for.
    :param gene_file_path: The path of the gene file
    :param file_separator: The separator used within the file. This can be 'comma', 'tab', or 'semicolon'.  By default,
    the file separator is set to csv.
    :param is_uniprot: A boolean value indicating whether the given gene list or file containing uniprot ids or HGNC
    symbols. By default, the value is set to False indicating that a "symbol" column is present with the respective
    HGNC symbols. If set to True, the file with "uniprot" column is expected.
    """

    # Load chembl target mapper files
    chembl_mapper = pd.read_csv(
        f"{MAPPER_DIR}/chembl_uniprot_mapping.txt",
        dtype=str,
        skiprows=1,
        sep="\t",
        names=["uniprot", "chembl_id", "name", "type"],
    )
    chembl_mapper = chembl_mapper[["uniprot", "chembl_id"]]
    chembl_mapper.set_index("uniprot", inplace=True)
    chembl_mapper = chembl_mapper.to_dict()["chembl_id"]

    hgnc_mapper = pd.read_csv(
        f"{MAPPER_DIR}/hgnc_mapper.tsv", sep="\t", index_col="Approved symbol"
    ).to_dict()["UniProt ID(supplied by UniProt)"]

    # Loop to get and store the genes-chemical information from ChEMBL
    if os.path.exists(f"{MAPPER_DIR}/{analysis_name}_gene_to_chemicals.json"):
        gene_chemical_dict = json.load(
            open(f"{MAPPER_DIR}/{analysis_name}_gene_to_chemicals.json")
        )
    else:
        gene_chemical_dict = defaultdict()

    new_count = 0

    # Extract the gene
    if file_separator == "comma":
        _separator = ","
    elif file_separator == "semicolon":
        _separator = ";"
    else:
        assert file_separator == "tab"
        _separator = "\t"

    if gene_file_path:
        df = pd.read_csv(gene_file_path, sep=_separator)
        if is_uniprot:
            if "uniprot" not in list(df.columns):
                raise ValueError(
                    f'Please rename columns to : "uniprot" in case of uniprot id or "symbol" in case of HGNC symbols'
                )
        elif not is_uniprot:
            if "symbol" not in list(df.columns):
                raise ValueError(
                    f'Please rename columns to : "uniprot" in case of uniprot id or "symbol" in case of HGNC symbols'
                )

    if gene_file_path and _separator and is_uniprot:
        proteins = set(pd.read_csv(gene_file_path, sep=_separator)["uniprot"].tolist())
    elif gene_file_path and _separator and not is_uniprot:
        proteins = set(pd.read_csv(gene_file_path, sep=_separator)["symbol"].tolist())
    else:
        proteins = gene_list

    # Loop to get chemicals related to target
    for identifier in tqdm(proteins, desc="Extracting chemicals for targets"):
        if identifier in gene_chemical_dict:
            continue

        new_count += 1
        # Extract chemical-target data from ChEMBL
        chemical_list = target_to_chemical(
            protein=identifier,
            protein_mapping=hgnc_mapper,
            chemical_mapping=chembl_mapper,
            is_uniprot=is_uniprot,
        )
        gene_chemical_dict[identifier] = chemical_list

    # Save dict for re-use
    if new_count > 0:
        with open(f"{MAPPER_DIR}/{analysis_name}_gene_to_chemicals.json", "w") as f:
            json.dump(gene_chemical_dict, f, ensure_ascii=False, indent=2)

    # Get genes with no chemical hits
    get_chemical_overview(f"{MAPPER_DIR}/{analysis_name}_gene_to_chemicals.json")

    return gene_chemical_dict



# -*- coding: utf-8 -*-

import logging
from typing import Dict, Optional
from urllib.error import URLError

import pandas as pd
from pubchempy import get_compounds

logger = logging.getLogger()
logging.basicConfig(level=logging.INFO)
pubchempy_logger = logging.getLogger("pubchempy")
pubchempy_logger.setLevel(logging.WARNING)

"""Protein mapper functions"""


def get_hgnc_id() -> Dict[str, str]:
    """Mapping dictionary for HGNC symbol to HGNC identifiers"""
    protein_mapping = pd.read_csv(
        f"https://www.genenames.org/cgi-bin/download/custom?col=gd_hgnc_id&col=gd_status&col=md_prot_id&status=Approved&hgnc_dbtag=on&order_by=gd_app_sym_sort&format=text&submit=submit",
        sep="\t",
        index_col="Approved symbol",
    ).to_dict()["HGNC ID"]

    return protein_mapping


def hgnc_to_chembl(
    chemical_mapper: Dict[str, str], uniprot_mapper: Dict[str, str], hgnc_symbol: str
) -> Optional[str]:
    """Mapping HGNC symbol to ChEMBL identifiers.

    :param chemical_mapper: A dictionary mapping the UNIPROT identifiers to ChEMBL
    :param uniprot_mapper: A dictionary mapping the HGNC identifiers to UNIPROT
    :param hgnc_symbol: A HGNC symbol
    """
    uniprot_id = uniprot_mapper.get(hgnc_symbol)
    return uniprot_to_chembl(chemical_mapper=chemical_mapper, uniprot_id=uniprot_id)


def uniprot_to_chembl(chemical_mapper: dict, uniprot_id: str) -> Optional[str]:
    """Mapping UniProt identifiers to ChEMBL identifiers.

    :param chemical_mapper: A dictionary mapping the UNIPROT identifiers to ChEMBL
    :param uniprot_id: UNIPROT identifier of a protein
    """
    target_chembl = chemical_mapper.get(uniprot_id)
    return target_chembl


"""Chemical mapper functions"""


def get_chemical_names(chembl_id: str) -> str:
    """Method to get chemical name from ChEMBL id.

    :param chembl_id: ChEMBL identifier of a compound
    """
    try:
        chemical_name = get_compounds(chembl_id, "name")[0].synonyms[0]
    except (IndexError, URLError):
        chemical_name = chembl_id
    return chemical_name



# -*- coding: utf-8 -*-

"""Script for harmonizing the ChEMBL chemicals with patent chemicals."""

import json
import logging
import os
from collections import defaultdict

import pandas as pd
from pubchempy import get_synonyms
from tqdm import tqdm

from pemt.constants import MAPPER_DIR, PATENT_DIR
from pemt.utils import get_chemical_names

logger = logging.getLogger(__name__)


def get_surechembl_id(
    chemical_id: str, chemical_name: str, chemical_mapper: dict
) -> str:
    """Method to get SureChEMBL identifier of chemicals.

    :param chemical_id: ChEMBL identifier of the chemical
    :param chemical_name: Name of the chemical
    :param chemical_mapper: Dictionary of identifier mapping between ChEMBL and SureChEMBL
    """
    surechembl_id = chemical_mapper.get(chemical_id)

    if surechembl_id:
        return surechembl_id

    try:
        synm_dict = get_synonyms(chemical_name, namespace="name")[0]
    except IndexError:
        return None

    try:
        surechembl_id = [
            synonym for synonym in synm_dict["Synonym"] if synonym.startswith("SCHEMBL")
        ][0]
    except IndexError:
        return None

    return surechembl_id


def harmonize_chemicals(analysis_name: str, from_genes: bool = True) -> None:
    """Method that allows mapping from ChEMBL to SureChEMBL identifiers.

    :param analysis_name: The name of the analysis you want to run. This name would be used to save the resultant file.
    :param from_genes: Boolean indicating where the process needs to get chemicals based on genes or not.
    """

    # Load cached data if it exists
    if os.path.exists(f"{PATENT_DIR}/{analysis_name}_chemicals.tsv"):
        chemical_df = pd.read_csv(
            f"{PATENT_DIR}/{analysis_name}_chemicals.tsv",
            sep="\t",
            dtype=str,
        )

        cache_dict = pd.read_csv(
            f"{PATENT_DIR}/{analysis_name}_chemicals.tsv",
            sep="\t",
            dtype=str,
            index_col="chembl",
        ).to_dict()["schembl_id"]

    else:
        chemical_df = pd.DataFrame(columns=["chembl", "schembl_id", "name"])
        cache_dict = {}

    # Load chembl - schembl mapper
    with open(f"{MAPPER_DIR}/chemical_mapper.json") as f:
        chemical_mapper = json.load(f)

    # Add caching system
    cache = 0
    genes_skipped = 0

    if os.path.exists(f"{MAPPER_DIR}/{analysis_name}_chemical_names.json"):
        chemical_names = json.load(
            open(f"{MAPPER_DIR}/{analysis_name}_chemical_names.json")
        )
    else:
        chemical_names = defaultdict(str)

    if from_genes:
        if os.path.exists(f"{MAPPER_DIR}/{analysis_name}_gene_to_chemicals.json"):
            gene_chemical_dict = json.load(
                open(f"{MAPPER_DIR}/{analysis_name}_gene_to_chemicals.json")
            )
        else:
            raise FileNotFoundError(
                f"Please ensure that you run the experimental data extractor file first."
            )

        for genes in tqdm(
            gene_chemical_dict, desc="Harmonizing chemicals for patent retrieval"
        ):
            # Extract chemical-target data from ChEMBL
            chemical_list = gene_chemical_dict[genes]

            if len(chemical_list) < 1:
                genes_skipped += 1
                continue

            for chembl_id in chemical_list:
                # Get name for chemical and store in dict
                if chembl_id not in chemical_names:
                    chemical_names[chembl_id] = get_chemical_names(chembl_id)

                # Store chembl to surechembl mapping in separate file
                if chembl_id not in cache_dict:
                    surechembl_id = get_surechembl_id(
                        chemical_id=chembl_id,
                        chemical_name=chemical_names[chembl_id],
                        chemical_mapper=chemical_mapper,
                    )

                    if not surechembl_id:
                        continue

                    cache += 1  # add new data only
                    cache_dict[chembl_id] = surechembl_id

                    tmp_df = pd.DataFrame(
                        {
                            "chembl": chembl_id,
                            "schembl_id": surechembl_id,
                            "name": chemical_names[chembl_id],
                        },
                        index=[0],
                    )

                    chemical_df = pd.concat([chemical_df, tmp_df], ignore_index=True)

                    if cache == 10:
                        chemical_df.to_csv(
                            f"{PATENT_DIR}/{analysis_name}_chemicals.tsv",
                            sep="\t",
                            index=False,
                        )
                        cache = 0

                        # Save chemical mapping dict for re-use
                        with open(
                            f"{MAPPER_DIR}/{analysis_name}_chemical_names.json", "w"
                        ) as f:
                            json.dump(chemical_names, f, ensure_ascii=False, indent=2)
    else:

        for _, row in tqdm(
            chemical_df.iterrows(),
            total=chemical_df.shape[0],
            desc="Harmonzing chemicals for patent retrival",
        ):
            chembl_id = row["chembl"]
            # Get name for chemical and store in dict
            if chembl_id not in chemical_names:
                chemical_names[chembl_id] = get_chemical_names(chembl_id)

            # Store chembl to surechembl mapping in separate file
            if chembl_id not in cache_dict:
                surechembl_id = get_surechembl_id(
                    chemical_id=chembl_id,
                    chemical_name=chemical_names[chembl_id],
                    chemical_mapper=chemical_mapper,
                )

                if not surechembl_id:
                    continue

                cache += 1  # add new data only
                cache_dict[chembl_id] = surechembl_id

                tmp_df = pd.DataFrame(
                    {
                        "chembl": chembl_id,
                        "schembl_id": surechembl_id,
                        "name": chemical_names[chembl_id],
                    },
                    index=[0],
                )

                chemical_df = pd.concat([chemical_df, tmp_df], ignore_index=True)

                if cache == 10:
                    chemical_df.to_csv(
                        f"{PATENT_DIR}/{analysis_name}_chemicals.tsv",
                        sep="\t",
                        index=False,
                    )
                    cache = 0

                    # Save chemical mapping dict for re-use
                    with open(
                        f"{MAPPER_DIR}/{analysis_name}_chemical_names.json", "w"
                    ) as f:
                        json.dump(chemical_names, f, ensure_ascii=False, indent=2)

    chemical_df.dropna(subset=["schembl_id"], inplace=True)

    chemical_df.to_csv(
        f"{PATENT_DIR}/{analysis_name}_chemicals.tsv", sep="\t", index=False
    )

    with open(f"{MAPPER_DIR}/{analysis_name}_chemical_names.json", "w") as f:
        json.dump(chemical_names, f, ensure_ascii=False, indent=2)



# -*- coding: utf-8 -*-

import os
import re
import sys

sys.path.insert(0, os.path.abspath("../../src"))

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.intersphinx",
    "sphinx.ext.todo",
    "sphinx.ext.coverage",
    "sphinx.ext.viewcode",
    "sphinx_click.ext",
]
templates_path = ["_templates"]
source_suffix = ".rst"
master_doc = "index"
project = "PEMT"
copyright = "2022-2025, Yojana Gadiya and Andrea Zaliani"
author = "Yojana Gadiya and Andrea Zaliani"

release = "0.0.2"

parsed_version = re.match(
    "(?P<major>\d+)\.(?P<minor>\d+)\.(?P<patch>\d+)(?:-(?P<release>[0-9A-Za-z-]+(?:\.[0-9A-Za-z-]+)*))?(?:\+(?P<build>[0-9A-Za-z-]+(?:\.[0-9A-Za-z-]+)*))?",
    release,
)
version = parsed_version.expand("\g<major>.\g<minor>.\g<patch>")

tags = set()

if parsed_version.group("release"):
    tags.add("prerelease")

language = None
exclude_patterns = []
pygments_style = "sphinx"
todo_include_todos = True
html_theme = "sphinx_rtd_theme"
html_static_path = []
htmlhelp_basename = "pemtdoc"
latex_elements = {}
latex_documents = [
    (
        master_doc,
        "pemt.tex",
        "PEMT Documentation",
        "Yojana Gadiya and Andrea Zaliani",
        "manual",
    ),
]
man_pages = [(master_doc, "pemt", "PEMT Documentation", [author], 1)]
texinfo_documents = [
    (master_doc, "pemt", "PEMT Documentation", author, "PEMT"),
]

# Example configuration for intersphinx: refer to the Python standard library.
intersphinx_mapping = {
    "python": ("https://docs.python.org/3", None),
}

autodoc_member_order = "bysource"
autoclass_content = "both"

if os.environ.get("READTHEDOCS"):
    tags.add("readthedocs")



# -*- coding: utf-8 -*-

"""File containing all constant required for the package."""

import os

"""File paths."""
HERE = os.path.dirname(os.path.realpath(__file__))
DATA_DIR = os.path.join(HERE, "../../data")
PATENT_DIR = os.path.join(DATA_DIR, "patent_dumps")
MAPPER_DIR = os.path.join(DATA_DIR, "mapper")

"""Valid IPC codes."""
VALID_CODES = {
    "A61B",
    "A61C",
    "A61D",
    "A61P",
    "A61Q",
    "A23B",
    "A23C",
    "A23D",
    "A23F",
    "A23G",
    "A23J",
    "A23L",
    "A23N",
    "A23P",
    "C01B",
    "C01C",
    "C01G",
    "C12P",
    "C12Q",
    "C07B",
    "C07C",
    "C07D",
    "C07F",
    "C07G",
    "C07H",
    "C07J",
    "C07K",
    "C08B",
    "C08C",
    "C08F",
    "C08G",
    "C08H",
    "C08J",
    "C08K",
    "C08L",
}

"""Country code to country name mapper from EPO."""
COUNTRY_MAPPER = {
    "AL": "Albania",
    "AP": "African Regional Industrial Property Organization",
    "AR": "Argentina",
    "AT": "Austria",
    "AU": "Australia",
    "BA": "Bosnia and Herzegovina",
    "BE": "Belgium",
    "BG": "Bulgaria",
    "BR": "Brazil",
    "CA": "Canada",
    "CH": "Switzerland",
    "CL": "Chile",
    "CN": "China",
    "CO": "Colombia",
    "CR": "Costa Rica",
    "CS": "Czechoslovakia",
    "CU": "Cuba",
    "CY": "Cyprus",
    "CZ": "Czech Republic",
    "DD": "Germany",
    "DE": "Germany",
    "DK": "Denmark",
    "DZ": "Algeria",
    "EA": "Eurasian Patent Organisation",
    "EC": "Ecuador",
    "EE": "Estonia",
    "EG": "Egypt",
    "EP": "European Patent Office",
    "ES": "Spain",
    "FI": "Finland",
    "FR": "France",
    "GB": "United Kingdom",
    "GC": "Gulf Cooperation Council",
    "GE": "Georgia",
    "GR": "Greece",
    "GT": "Guatemala",
    "HK": "Hong Kong (China)",
    "HR": "Croatia",
    "HU": "Hungary",
    "ID": "Indonesia",
    "IE": "Ireland",
    "IL": "Israel",
    "IN": "India",
    "IS": "Iceland",
    "IT": "Italy",
    "JP": "Japan",
    "KE": "Kenya",
    "KR": "Republic of Korea",
    "LI": "Liechtenstein",
    "LT": "Lithuania",
    "LU": "Luxembourg",
    "LV": "Latvia",
    "MA": "Morocco",
    "MC": "Monaco",
    "MD": "Republic of Moldova",
    "MK": "Republic of North Macedonia",
    "MN": "Mongolia",
    "MT": "Malta",
    "MW": "Malawi",
    "MX": "Mexico",
    "MY": "Malaysia",
    "NC": "New Caledonia",
    "NI": "Nicaragua",
    "NL": "Netherlands",
    "NO": "Norway",
    "NZ": "New Zealand",
    "OA": "African Intellectual Property Organization",
    "PA": "Panama",
    "PE": "Peru",
    "PH": "Philippines",
    "PL": "Poland",
    "PT": "Portugal",
    "RO": "Romania",
    "RU": "Russian Federation",
    "SE": "Sweden",
    "SG": "Singapore",
    "SI": "Slovenia",
    "SK": "Slovakia",
    "SU": "Union of Soviet Socialist Republics (USSR)",
    "SV": "El Salvador",
    "TJ": "Tajikistan",
    "TR": "Turkey",
    "TT": "Trinidad and Tobago",
    "TW": "Chinese Taipei",
    "UA": "Ukraine",
    "US": "United States of America",
    "UY": "Uruguay",
    "VN": "Vietnam",
    "WO": "World Intellectual Property Organisation (WIPO)",
    "YU": "Yugoslavia (Serbia and Montenegro)",
    "ZA": "South Africa",
    "ZM": "Zambia",
    "ZW": "Zimbabwe",
}



# -*- coding: utf-8 -*-

"""Tests for CDF patient incorporation."""

import os
import unittest

from pemt.chemical_extractor.experimental_data_extraction import extract_chemicals
from pemt.constants import MAPPER_DIR

TEST_FOLDER = os.path.dirname(os.path.realpath(__file__))
DUMMY_DATA = os.path.join(TEST_FOLDER, 'resources', 'dummy_gene.tsv')


class TestInput(unittest.TestCase):
    """Tests for input method."""

    def test_chemical_extractor(self):
        """Test chemical extractor."""

        output = extract_chemicals(
            analysis_name='test',
            gene_file_path=DUMMY_DATA,
            file_separator='comma',
            is_uniprot=True
        )

        """Test dummy protein are returned"""
        self.assertEqual(
            len(output),
            11
        )

        """Test dummy protein are outputted in a file"""
        self.assertTrue(os.path.exists(f'{MAPPER_DIR}/test_gene_to_chemicals.json'))


# -*- coding: utf-8 -*-

"""Command line interface."""

import json
import logging
from collections import defaultdict

import click
import numpy as np
import pandas as pd
from tqdm import tqdm

from pemt.chemical_extractor.experimental_data_extraction import extract_chemicals
from pemt.constants import MAPPER_DIR, PATENT_DIR
from pemt.patent_extractor.patent_chemical_harmonizer import harmonize_chemicals
from pemt.patent_extractor.patent_enrichment import extract_patent

logger = logging.getLogger(__name__)


@click.group()
def main():
    """Run PEMT."""
    logging.basicConfig(format="%(asctime)s - %(levelname)s - %(name)s - %(message)s")


input_data = click.option(
    "--data",
    help="Path to tab-separated gene data file",
    type=click.Path(file_okay=True, dir_okay=False, exists=True),
    required=True,
)
input_data_type = click.option(
    "--input-type",
    help="Type of data file i.e. 'tab' for tsv or 'comma' for csv files",
    type=str,
    default=",",
)
analysis_name = click.option(
    "--name",
    help="Name of the analysis that is to be run",
    type=str,
    required=True,
)
has_uniprot = click.option(
    "--uniprot/--no-uniprot",
    default=True,
    help="Boolean value indicating whether the gene data file has uniprot ids or not.",
)
system_name = click.option(
    "--os",
    type=click.Choice(["linux", "mac", "windows"], case_sensitive=False),
    help="The OS system on which is the script is running",
    multiple=False,
)
chromedriver_path = click.option(
    "--chromedriver-path",
    help="The path where the chromedriver can be found on the users computer",
    type=str,
    required=True,
)
patent_year = click.option(
    "--year",
    help="The year from which you want to retrive patents from",
    type=int,
    default=2000,
)
from_chemical = click.option(
    "--chemical/--no-chemical",
    default=True,
    help="Boolean value indicating whether the chemical data is provided by the user or not",
)
chemcial_data = click.option(
    "--chemical-data",
    help="Path to tab-separated chemical data file with single column of chembl_ids",
    type=click.Path(),
    default="",
)


@main.command(help="Extract chemicals for genes of interest")
@analysis_name
@input_data
@input_data_type
@has_uniprot
def run_chemical_extractor(
    name: str, data: str, input_type: str, uniprot: bool
) -> None:
    """Extracting chemicals for genes with experiemtal data."""
    click.echo(f"Starting the chemical extractor pipeline for {name}")

    if uniprot:
        with_uniprot = True
    else:
        with_uniprot = False

    gene_chemical_dict = extract_chemicals(
        analysis_name=name,
        gene_file_path=data,
        file_separator=input_type,
        is_uniprot=with_uniprot,
    )

    click.echo(
        f"Completed the chemical extractor pipeline for {len(gene_chemical_dict)} genes."
    )
    click.echo(f"Data file can be found under {MAPPER_DIR}")


@main.command(help="Extract patent for filtered chemicals")
@analysis_name
@system_name
@chromedriver_path
@patent_year
@from_chemical
@chemcial_data
def run_patent_extractor(
    name: str,
    os: str,
    chromedriver_path: str,
    year: str,
    chemical: bool,
    chemical_data: str,
) -> None:
    """Extracting patent from chemical data."""
    click.echo(f"Starting to pre-process the chemical data for patent retrieval")

    if chemical:
        df = pd.read_csv(chemical_data, sep="\t", dtype=str)

        df.to_csv(f"{PATENT_DIR}/{name}_chemicals.tsv", sep="\t", index=False)

        harmonize_chemicals(analysis_name=name, from_genes=False)
    else:
        harmonize_chemicals(analysis_name=name)

    click.echo(f"Starting the patent extractor pipeline for {name}")

    patent_df = extract_patent(
        analysis_name=name,
        chrome_driver_path=chromedriver_path,
        os_system=os,
        patent_year=year,
    )

    if patent_df.empty:
        click.echo(f"No patents found!")
        return None

    # Since the original patent data has chemical with no patents, we remove those entries from the data
    patent_df = patent_df[~patent_df["patent_id"].isna()]
    patent_df.to_csv(
        f"{PATENT_DIR}/cleaned_{name}_patent_data.tsv", sep="\t", index=False
    )

    if chemical:
        click.echo(f"Done with retrival of patents")
        click.echo(f"Data file can be found under {PATENT_DIR}")
        return

    gene_chemical_data = json.load(open(f"{MAPPER_DIR}/{name}_gene_to_chemicals.json"))
    chemical_to_gene_mapper = defaultdict(set)

    for gene, chemicals in tqdm(gene_chemical_data.items()):
        for chemical in chemicals:
            chemical_to_gene_mapper[chemical].add(gene)

    patent_df["genes"] = patent_df["chembl"].map(
        lambda x: ", ".join(chemical_to_gene_mapper[x])
    )

    patent_df.to_csv(f"{PATENT_DIR}/{name}_gene_patent_data.tsv", sep="\t", index=False)

    click.echo(f"Done with retrival of patents")
    click.echo(f"Data file can be found under {PATENT_DIR}")


@main.command(help="Run the PEMT tool with gene data")
@analysis_name
@input_data
@input_data_type
@has_uniprot
@chromedriver_path
@system_name
@patent_year
def run_pemt(
    name: str,
    data: str,
    input_type: str,
    uniprot: bool,
    chromedriver_path: str,
    os: str,
    year: str,
) -> None:
    """Runs the PEMT tool with all the components together."""
    click.echo(f"Starting to run PEMT workflow for {name}")

    click.echo(f"Running the chemical extractor pipeline")

    if uniprot:
        with_uniprot = True
    else:
        with_uniprot = False

    gene_chemical_dict = extract_chemicals(
        analysis_name=name,
        gene_file_path=data,
        file_separator=input_type,
        is_uniprot=with_uniprot,
    )

    click.echo(
        f"Completed running the chemical extractor pipeline for {len(gene_chemical_dict)} genes."
    )

    click.echo(f"Ppre-processing the chemical data for patent retrieval")

    harmonize_chemicals(analysis_name=name)

    click.echo(f"Running the patent extractor pipeline")

    patent_df = extract_patent(
        analysis_name=name,
        chrome_driver_path=chromedriver_path,
        os_system=os,
        patent_year=year,
    )

    if patent_df.empty:
        click.echo(f"No patents found!")
        return None

    # Since the original patent data has chemical with no patents, we remove those entries from the data
    patent_df = patent_df[~patent_df["patent_id"].isna()]
    patent_df.to_csv(
        f"{PATENT_DIR}/cleaned_{name}_patent_data.tsv", sep="\t", index=False
    )

    chemical_to_gene_mapper = defaultdict(set)

    for gene, chemicals in tqdm(gene_chemical_data.items()):
        for chemical in chemicals:
            chemical_to_gene_mapper[chemical].add(gene)

    patent_df["genes"] = patent_df["chembl"].map(
        lambda x: ", ".join(chemical_to_gene_mapper[x])
    )

    patent_df.to_csv(f"{PATENT_DIR}/{name}_gene_patent_data.tsv", sep="\t", index=False)

    click.echo(f"Done with retrival of patents")


if __name__ == "__main__":
    main()
